"""add WatchEvent.stopped_at column

Revision ID: 1f69c57af9a
Revises: 26134e26286
Create Date: 2014-09-16 02:32:38.448495

"""

# revision identifiers, used by Alembic.
revision = '1f69c57af9a'
down_revision = '26134e26286'

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('watch_event', sa.Column('stopped_at', sa.Float(), nullable=True))
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('watch_event', 'stopped_at')
    ### end Alembic commands ###
